import 'effect.dart';
import 'functions.dart';

class AsyncBuilder<TArgs, TResult> {
  final Future<TResult> Function(TArgs) _run;

  AsyncBuilder._(this._run);

  AsyncBuilder<TArgs, T> map<T>(MapFun<TArgs, TResult, T> map) {
    final fun = (TArgs a) async {
      final res = await _run(a);
      return map(a, res);
    };
    return AsyncBuilder._(fun);
  }

  AsyncBuilder<TArgs, TResult> mapException(
      ExceptionMapFun<TArgs, TResult> mapException) {
    final fun = (TArgs args) async {
      try {
        return await _run(args);
      } catch (e, t) {
        return mapException(args, e, t);
      }
    };
    return AsyncBuilder._(fun);
  }

  AsyncBuilder<TArgs, TResult> onData(OnDataEffect<TArgs, TResult> effect) {
    final fun = (TArgs args) async {
      final res = await _run(args);

      //ignore: unawaited_futures
      effect.onData(args, res);
      return res;
    };
    return AsyncBuilder._(fun);
  }

  AsyncBuilder<TArgs, TResult> onException(
      OnExceptionEffect<TArgs, TResult> effect) {
    final fun = (TArgs args) async {
      try {
        return await _run(args);
      } catch (e, t) {
        //ignore: unawaited_futures
        effect.onException(args, e, t);
        rethrow;
      }
    };
    return AsyncBuilder._(fun);
  }

  Future<TResult> run(TArgs args) {
    return _run(args);
  }
}

class AsyncAction<TArgs, TResult> extends AsyncBuilder<TArgs, TResult> {
  AsyncAction(CallFun<TArgs, TResult> fun) : super._(fun);
}
