import 'package:meta/meta.dart';
import 'package:rxdart/rxdart.dart';

abstract class Emitter<TResult> {
  const Emitter();

  Observable<TResult> get stream;

  @mustCallSuper
  void dispose() {}
}

class SubjectEmitter<TResult, TSubject extends Subject<TResult>>
    extends Emitter<TResult> {
  const SubjectEmitter(this.subject);
  @protected
  final TSubject subject;

  void safeAdd(TResult event) {
    if (!subject.isClosed) {
      subject.add(event);
    }
  }

  void safeAddException(dynamic e, [StackTrace t]) {
    if (!subject.isClosed) {
      subject.addError(e, t);
    }
  }

  @override
  Observable<TResult> get stream => subject;

  @override
  void dispose() {
    subject.close();
    super.dispose();
  }
}

class PublishSubjectEmitter<TResult>
    extends SubjectEmitter<TResult, PublishSubject<TResult>> {
  PublishSubjectEmitter({void onListen(), void onCancel(), bool sync = false})
      : super(PublishSubject(
          onListen: onListen,
          onCancel: onCancel,
          sync: sync,
        ));
}

class BehaviorSubjectEmitter<TResult>
    extends SubjectEmitter<TResult, BehaviorSubject<TResult>> {
  BehaviorSubjectEmitter({void onListen(), void onCancel(), bool sync = false})
      : super(BehaviorSubject(
          onListen: onListen,
          onCancel: onCancel,
          sync: sync,
        ));
}
