import 'async_builder.dart';

abstract class Action<TResult> {
  const Action();
}

abstract class Repository {
  Repository() : _asyncBuilders = {};
  final Map<Type, AsyncBuilder> _asyncBuilders;

  void addAsyncBuilder<TArgs, TResult>(AsyncBuilder<TArgs, TResult> builder) {
    final type = _typeOf<TArgs>();
    _asyncBuilders[type] = builder;
  }

  Future<TResult> execute<TResult>(Action<TResult> args) {
    final type = args.runtimeType;
    final builder = _asyncBuilders[type];

    if (builder == null) {
      throw AsyncBuilderNotFound(type);
    }

    return builder.run(args);
  }

  void dispose() {}
}

Type _typeOf<T>() => T;

class AsyncBuilderNotFound implements Exception {
  const AsyncBuilderNotFound(this.type);
  final Type type;
}
