import 'package:rxdart/rxdart.dart';

import 'effect.dart';
import 'emitter.dart';

abstract class ResultEmitter<TArgs, TResult, TSubject extends Subject<TResult>>
    extends SubjectEmitter<TResult, TSubject>
    implements AsyncEffect<TArgs, TResult> {
  const ResultEmitter(TSubject subject) : super(subject);

  Future<void> onException(TArgs args, dynamic e, [StackTrace t]) async {
    safeAddException(e, t);
  }

  Future<void> onData(TArgs args, TResult res) async {
    safeAdd(res);
  }
}

class PublishResultEmitter<TArgs, TResult>
    extends ResultEmitter<TArgs, TResult, PublishSubject<TResult>> {
  PublishResultEmitter({void onListen(), void onCancel(), bool sync = false})
      : super(PublishSubject(
          onListen: onListen,
          onCancel: onCancel,
          sync: sync,
        ));
}

class BehaviorResultEmitter<TArgs, TResult>
    extends ResultEmitter<TArgs, TResult, BehaviorSubject<TResult>> {
  BehaviorResultEmitter({void onListen(), void onCancel(), bool sync = false})
      : super(BehaviorSubject(
          onListen: onListen,
          onCancel: onCancel,
          sync: sync,
        ));
}
