import 'package:flutter_test/flutter_test.dart';
import 'package:frebu/frebu.dart';
import 'package:mockito/mockito.dart';
import 'package:rxdart/rxdart.dart';

class FakeSubject<T> extends Mock implements Subject<T> {}

void main() {
  group('SubjectEmitter invokes correct methods from subject', () {
    FakeSubject<int> subject;
    SubjectEmitter<int, FakeSubject<int>> emitter;

    setUp(() {
      subject = FakeSubject<int>();
      emitter = SubjectEmitter<int, FakeSubject<int>>(subject);
    });

    tearDown(() {
      subject.close();
    });

    test('safeAdd invokes add method', () {
      when(subject.isClosed).thenReturn(false);

      emitter.safeAdd(3);
      emitter.safeAdd(5);
      emitter.safeAdd(6);
      emitter.safeAdd(9);

      verify(subject.add(any)).called(4);
    });

    test('safeException invokes addError method', () {
      when(subject.isClosed).thenReturn(false);

      emitter.safeAddException("exception");
      emitter.safeAddException("exception2");

      verify(subject.addError(any)).called(2);
    });

    test('safeAdd does not invoke internall methods after subject closing', () {
      when(subject.isClosed).thenReturn(true);

      emitter.safeAdd(1);
      emitter.safeAdd(2);

      verifyNever(subject.add(any));
    });

    test(
        'safeAddException does not invoke internal methods after subject closing',
        () {
      when(subject.isClosed).thenReturn(true);

      emitter.safeAddException('exception');
      emitter.safeAddException('exception2');

      verifyNever(subject.addError(any));
      verifyNever(subject.addError(any, any));
    });

    test('dispose closes the subject', () {
      emitter.dispose();

      verify(subject.close());
    });
  });

  group('Test stream from subjectemitter', () {
    test('stream emits events', () {
      final subject = PublishSubject<int>();
      final emitter = SubjectEmitter<int, PublishSubject<int>>(subject);

      expectLater(emitter.stream, emitsInOrder([3, 5, 6, 9]));

      emitter.safeAdd(3);
      emitter.safeAdd(5);
      emitter.safeAdd(6);
      emitter.safeAdd(9);

      subject.close();
    });
  });
}
