import 'package:flutter_test/flutter_test.dart';
import 'package:frebu/src/event_bus.dart';
import 'package:mockito/mockito.dart';

class A extends Event {}

class B extends A {}

class AListener {
  void a(A a) {}
}

class MockAListener extends Mock implements AListener {}

void main() {
  group('EventBus tests', () {
    EventBus bus;
    MockAListener fakeAListener;

    setUp(() {
      bus = EventBus();
      fakeAListener = MockAListener();
    });

    tearDown(() {
      bus.dispose();
    });

    test('Emit method emits events to stream', () {
      expectLater(
          bus.eventsStream,
          emitsInOrder([
            isA<A>(),
            isA<B>(),
          ]));

      bus.emit(A());
      bus.emit(B());
    });

    test('listener gets events only when one is subscribing', () async {
      bus.emit(A());

      bus.addListener(fakeAListener.a);
      bus.emit(A());
      await untilCalled(fakeAListener.a(any));

      bus.emit(B());
      await untilCalled(fakeAListener.a(any));

      bus.removeListener(fakeAListener.a);

      verify(fakeAListener.a(any)).called(2);
    });
  });
}
