import 'package:flutter_test/flutter_test.dart';
import 'package:frebu/frebu.dart';

class A extends Action<int> {}

class B extends Action<int> {}

class C extends A {}

class ARepository extends Repository {
  ARepository() {
    addAsyncBuilder(AsyncAction((A a) async => 1));
  }
}

void main() {
  group('Repository tests', () {
    ARepository repository;

    setUp(() {
      repository = ARepository();
    });

    tearDown(() {
      repository.dispose();
    });

    test('Repository finds builder by type of event', () async {
      final result = await repository.execute(A());

      expect(result, 1);
    });

    test('Repository throws away exception when builder is not found', () async {
      expect(
        () => repository.execute(B()),
        throwsA(isA<AsyncBuilderNotFound>()),
      );
    });

    test('Repository finds builder only for exact type', () async {
      expect(
        () => repository.execute(C()),
        throwsA(isA<AsyncBuilderNotFound>()),
      );
    });
  });
}
